# Copyright (C) 2012, Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from sugar_stats import sniffers, dbus_monitor
from sugar_stats.collector import Collector


_collector = None


def start(backend, collector_class=Collector):
    global _collector
    if _collector is not None:
        return
    _collector = collector_class(sniffers.each(), backend)
    return _collector


def stop():
    global _collector
    if _collector is None:
        return
    _collector.close()
    _collector = None
    dbus_monitor.close()


if __name__ == '__main__':
    import sys
    import logging
    import gobject
    import dbus.mainloop.glib

    class _Collector(object):

        def __init__(self, sniffers_, *args):
            self.sniffers = {}
            for cls in sniffers_:
                self.sniffers[cls.__module__] = cls(self)

        def sniffer(self, name):
            return self.sniffers[name]

        def close(self):
            pass

        def update(self, stat, value, instance=None):
            postfix = ''
            if instance is not None:
                postfix = instance
            print '%-20s%-10s%s' % (stat, value, postfix)

    logging.basicConfig(level=logging.DEBUG)

    gobject.threads_init()
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

    if '-d' in sys.argv:
        collector_class_ = _Collector
        backend_ = None
    else:
        from sugar_stats.rrd import Rrd
        collector_class_ = Collector
        backend_ = Rrd('tmp', 3, ['RRA:AVERAGE:0.5:1:360'])

    start(backend_, collector_class_)

    mainloop = gobject.MainLoop()
    try:
        mainloop.run()
    except KeyboardInterrupt:
        pass
    finally:
        stop()
